@@ -68,7 +68,12 @@ class CinderVolume(vb.BaseVolume, sh.SchedulerHintsMixin):
         SIZE: properties.Schema(
             properties.Schema.INTEGER,
             _('The size of the volume in GB. '
-              'On update only increase in size is supported.'),
+              'On update only increase in size is supported. This property '
+              'is required unless property %(backup)s or %(vol)s or '
+              '%(snapshot)s is specified.')
+            % dict(backup=BACKUP_ID,
+                   vol=SOURCE_VOLID,
+                   snapshot=SNAPSHOT_ID),
             update_allowed=True,
             constraints=[
                 constraints.Range(min=1),
@@ -155,9 +160,9 @@ class CinderVolume(vb.BaseVolume, sh.SchedulerHintsMixin):
         ),
         MULTI_ATTACH: properties.Schema(
             properties.Schema.BOOLEAN,
-            _('Whether allow the volume to be attached more than once. '
-              'This property is only supported from Cinder API v2.'),
+            _('Whether allow the volume to be attached more than once.'),
             support_status=support.SupportStatus(version='6.0.0'),
+            default=False
         ),
     }
 
@@ -282,28 +287,39 @@ class CinderVolume(vb.BaseVolume, sh.SchedulerHintsMixin):
             return six.text_type(jsonutils.dumps(vol.metadata))
         elif name == self.METADATA_VALUES_ATTR:
             return vol.metadata
-        if cinder.volume_api_version >= 2:
-            if name == self.DISPLAY_NAME_ATTR:
-                return vol.name
-            elif name == self.DISPLAY_DESCRIPTION_ATTR:
-                return vol.description
+        if name == self.DISPLAY_NAME_ATTR:
+            return vol.name
+        elif name == self.DISPLAY_DESCRIPTION_ATTR:
+            return vol.description
         return six.text_type(getattr(vol, name))
 
-    # TODO(huangtianhua): remove this method when bug #1479641 is fixed.
-    def _show_resource(self):
-        volume = self.client().volumes.get(self.resource_id)
-        return volume._info
-
     def check_create_complete(self, vol_id):
         complete = super(CinderVolume, self).check_create_complete(vol_id)
         # Cinder just supports update read only for volume in available,
         # if we update in handle_create(), maybe the volume still in
         # creating, then cinder will raise an exception
         if complete:
+            self._store_config_default_properties()
             self._update_read_only(self.properties[self.READ_ONLY])
 
         return complete
 
+    def _store_config_default_properties(self, attributes=None):
+        """Method for storing default values of properties in resource data.
+
+        Some properties have default values, specified in project configuration
+        file, so cannot be hardcoded into properties_schema, but should be
+        stored for further using. So need to get created resource and take
+        required property's value.
+        """
+        if attributes is None:
+            attributes = self._show_resource()
+
+        if attributes.get('volume_type') is not None:
+            self.data_set(self.VOLUME_TYPE, attributes['volume_type'])
+        else:
+            self.data_delete(self.VOLUME_TYPE)
+
     def _extend_volume(self, new_size):
         try:
             self.client().volumes.extend(self.resource_id, new_size)
@@ -386,8 +402,8 @@ class CinderVolume(vb.BaseVolume, sh.SchedulerHintsMixin):
                            self.properties[self.NAME])
             update_description = (prop_diff.get(self.DESCRIPTION) or
                                   self.properties[self.DESCRIPTION])
-            kwargs = self._fetch_name_and_description(
-                cinder.volume_api_version, update_name, update_description)
+            kwargs = self._fetch_name_and_description(update_name,
+                                                      update_description)
             cinder.volumes.update(vol, **kwargs)
         # update the metadata for cinder volume
         if self.METADATA in prop_diff:
@@ -397,16 +413,10 @@ class CinderVolume(vb.BaseVolume, sh.SchedulerHintsMixin):
             cinder.volumes.update_all_metadata(vol, metadata)
         # retype
         if self.VOLUME_TYPE in prop_diff:
-            if cinder.volume_api_version == 1:
-                LOG.info(_LI('Volume type update not supported '
-                             'by Cinder API V1.'))
-                raise exception.NotSupported(
-                    feature=_('Using Cinder API V1, volume_type update'))
-            else:
-                if not vol:
-                    vol = cinder.volumes.get(self.resource_id)
-                new_vol_type = prop_diff.get(self.VOLUME_TYPE)
-                cinder.volumes.retype(vol, new_vol_type, 'never')
+            if not vol:
+                vol = cinder.volumes.get(self.resource_id)
+            new_vol_type = prop_diff.get(self.VOLUME_TYPE)
+            cinder.volumes.retype(vol, new_vol_type, 'never')
         # update read_only access mode
         if self.READ_ONLY in prop_diff:
             if not vol:
@@ -558,29 +568,30 @@ class CinderVolume(vb.BaseVolume, sh.SchedulerHintsMixin):
 
     def _build_exclusive_options(self):
         exclusive_options = []
+        allow_no_size_options = []
         if self.properties.get(self.SNAPSHOT_ID):
             exclusive_options.append(self.SNAPSHOT_ID)
+            allow_no_size_options.append(self.SNAPSHOT_ID)
         if self.properties.get(self.SOURCE_VOLID):
             exclusive_options.append(self.SOURCE_VOLID)
+            allow_no_size_options.append(self.SOURCE_VOLID)
         if self.properties.get(self.IMAGE):
             exclusive_options.append(self.IMAGE)
         if self.properties.get(self.IMAGE_REF):
             exclusive_options.append(self.IMAGE_REF)
-        return exclusive_options
+        return exclusive_options, allow_no_size_options
 
     def _validate_create_sources(self):
-        exclusive_options = self._build_exclusive_options()
+        exclusive_options, allow_no_size_ops = self._build_exclusive_options()
         size = self.properties.get(self.SIZE)
-        if size is None and len(exclusive_options) != 1:
+        if (size is None and
+                (len(allow_no_size_ops) != 1 or len(exclusive_options) != 1)):
             msg = (_('If neither "%(backup_id)s" nor "%(size)s" is '
-                     'provided, one and only one of '
-                     '"%(image)s", "%(image_ref)s", "%(source_vol)s", '
+                     'provided, one and only one of "%(source_vol)s", '
                      '"%(snapshot_id)s" must be specified, but currently '
                      'specified options: %(exclusive_options)s.')
                    % {'backup_id': self.BACKUP_ID,
                       'size': self.SIZE,
-                      'image': self.IMAGE,
-                      'image_ref': self.IMAGE_REF,
                       'source_vol': self.SOURCE_VOLID,
                       'snapshot_id': self.SNAPSHOT_ID,
                       'exclusive_options': exclusive_options})
@@ -604,19 +615,6 @@ class CinderVolume(vb.BaseVolume, sh.SchedulerHintsMixin):
         if res is not None:
             return res
 
-        # Scheduler hints are only supported from Cinder API v2
-        if (self.properties[self.CINDER_SCHEDULER_HINTS]
-                and self.client().volume_api_version == 1):
-            raise exception.StackValidationFailed(
-                message=_('Scheduler hints are not supported by the current '
-                          'volume API.'))
-        # Multi attach is only supported from Cinder API v2
-        if (self.properties[self.MULTI_ATTACH]
-                and self.client().volume_api_version == 1):
-            raise exception.StackValidationFailed(
-                message=_('Multiple attach is not supported by the current '
-                          'volume API. Use this property since '
-                          'Cinder API v2.'))
         # can not specify both image and imageRef
         image = self.properties.get(self.IMAGE)
         imageRef = self.properties.get(self.IMAGE_REF)
@@ -636,11 +634,33 @@ class CinderVolume(vb.BaseVolume, sh.SchedulerHintsMixin):
             self.IMAGE_REF, self.IMAGE, self.SOURCE_VOLID)
         props = dict(
             (key, value) for (key, value) in
-            six.iteritems(defn.properties(self.properties_schema))
+            self.properties.data.items()
             if key not in ignore_props and value is not None)
         props[self.BACKUP_ID] = backup_id
         return defn.freeze(properties=props)
 
+    def parse_live_resource_data(self, resource_properties, resource_data):
+        volume_reality = {}
+
+        if (resource_data.get(self.METADATA) and
+                resource_data.get(self.METADATA).get(
+                    self.READ_ONLY) is not None):
+            read_only = resource_data.get(self.METADATA).pop(self.READ_ONLY)
+            volume_reality.update({self.READ_ONLY: read_only})
+
+        old_vt = self.data().get(self.VOLUME_TYPE)
+        new_vt = resource_data.get(self.VOLUME_TYPE)
+        if old_vt != new_vt:
+            volume_reality.update({self.VOLUME_TYPE: new_vt})
+            self._store_config_default_properties(dict(volume_type=new_vt))
+
+        props_keys = [self.SIZE, self.NAME, self.DESCRIPTION,
+                      self.METADATA, self.BACKUP_ID]
+        for key in props_keys:
+            volume_reality.update({key: resource_data.get(key)})
+
+        return volume_reality
+
 
 class CinderVolumeAttachment(vb.BaseVolumeAttachment):
     """Resource for associating volume to instance.
@@ -689,7 +709,7 @@ class CinderVolumeAttachment(vb.BaseVolumeAttachment):
             # we still first detach the old resource so that
             # self.resource_id is not replaced prematurely
             volume_id = self.properties[self.VOLUME_ID]
-            server_id = self._stored_properties_data.get(self.INSTANCE_ID)
+            server_id = self.properties[self.INSTANCE_ID]
             self.client_plugin('nova').detach_volume(server_id,
                                                      self.resource_id)
             prg_detach = progress.VolumeDetachProgress(
