@@ -27,6 +27,8 @@ from heat.common.i18n import _
 from heat.common.i18n import _LI
 from heat.db import api as db_api
 from heat.engine import api
+from heat.engine import scheduler
+from heat.engine import software_config_io as swc_io
 from heat.objects import resource as resource_objects
 from heat.objects import software_config as software_config_object
 from heat.objects import software_deployment as software_deployment_object
@@ -41,27 +43,30 @@ class SoftwareConfigService(service.Service):
         sc = software_config_object.SoftwareConfig.get_by_id(cnxt, config_id)
         return api.format_software_config(sc)
 
-    def list_software_configs(self, cnxt, limit=None, marker=None,
-                              tenant_safe=True):
+    def list_software_configs(self, cnxt, limit=None, marker=None):
         scs = software_config_object.SoftwareConfig.get_all(
             cnxt,
             limit=limit,
-            marker=marker,
-            tenant_safe=tenant_safe)
+            marker=marker)
         result = [api.format_software_config(sc, detail=False) for sc in scs]
         return result
 
     def create_software_config(self, cnxt, group, name, config,
                                inputs, outputs, options):
 
+        swc_io.check_io_schema_list(inputs)
+        in_conf = [swc_io.InputConfig(**i).as_dict() for i in inputs]
+        swc_io.check_io_schema_list(outputs)
+        out_conf = [swc_io.OutputConfig(**o).as_dict() for o in outputs]
+
         sc = software_config_object.SoftwareConfig.create(cnxt, {
             'group': group,
             'name': name,
             'config': {
-                'inputs': inputs,
-                'outputs': outputs,
-                'options': options,
-                'config': config
+                rpc_api.SOFTWARE_CONFIG_INPUTS: in_conf,
+                rpc_api.SOFTWARE_CONFIG_OUTPUTS: out_conf,
+                rpc_api.SOFTWARE_CONFIG_OPTIONS: options,
+                rpc_api.SOFTWARE_CONFIG_CONFIG: config
             },
             'tenant': cnxt.tenant_id})
         return api.format_software_config(sc)
@@ -96,6 +101,7 @@ class SoftwareConfigService(service.Service):
         rs = db_api.resource_get_by_physical_resource_id(cnxt, server_id)
         if not rs:
             return
+        cnxt.session.refresh(rs)
         deployments = self.metadata_software_deployments(cnxt, server_id)
         md = rs.rsrc_metadata or {}
         md['deployments'] = deployments
@@ -117,10 +123,8 @@ class SoftwareConfigService(service.Service):
             requests.put(metadata_put_url, json_md)
         if metadata_queue_id:
             project = stack_user_project_id
-            token = self._get_user_token(cnxt, rs, project)
+            queue = self._get_zaqar_queue(cnxt, rs, project, metadata_queue_id)
             zaqar_plugin = cnxt.clients.client_plugin('zaqar')
-            zaqar = zaqar_plugin.create_for_tenant(project, token)
-            queue = zaqar.queue(metadata_queue_id)
             queue.post({'body': md, 'ttl': zaqar_plugin.DEFAULT_TTL})
 
     def _refresh_swift_software_deployment(self, cnxt, sd, deploy_signal_id):
@@ -169,24 +173,32 @@ class SoftwareConfigService(service.Service):
         return software_deployment_object.SoftwareDeployment.get_by_id(
             cnxt, sd.id)
 
-    def _get_user_token(self, cnxt, rs, project):
-        user = password = None
+    def _get_zaqar_queue(self, cnxt, rs, project, queue_name):
+        user = password = signed_url_data = None
         for rd in rs.data:
             if rd.key == 'password':
                 password = crypt.decrypt(rd.decrypt_method, rd.value)
             if rd.key == 'user_id':
                 user = rd.value
-        keystone = cnxt.clients.client('keystone')
-        return keystone.stack_domain_user_token(
-            user_id=user, project_id=project, password=password)
+            if rd.key == 'zaqar_queue_signed_url_data':
+                signed_url_data = jsonutils.loads(rd.value)
+        zaqar_plugin = cnxt.clients.client_plugin('zaqar')
+        if signed_url_data is None:
+            keystone = cnxt.clients.client('keystone')
+            token = keystone.stack_domain_user_token(
+                user_id=user, project_id=project, password=password)
+            zaqar = zaqar_plugin.create_for_tenant(project, token)
+        else:
+            signed_url_data.pop('project')
+            zaqar = zaqar_plugin.create_from_signed_url(project,
+                                                        **signed_url_data)
+
+        return zaqar.queue(queue_name)
 
     def _refresh_zaqar_software_deployment(self, cnxt, sd, deploy_queue_id):
-        rs = db_api.resource_get_by_physical_resource_id(cnxt, sd.server_id)
+        rs = db_api.resource_get_by_physical_resource_id(cnxt, sd.id)
         project = sd.stack_user_project_id
-        token = self._get_user_token(cnxt, rs, project)
-        zaqar_plugin = cnxt.clients.client_plugin('zaqar')
-        zaqar = zaqar_plugin.create_for_tenant(project, token)
-        queue = zaqar.queue(deploy_queue_id)
+        queue = self._get_zaqar_queue(cnxt, rs, project, deploy_queue_id)
 
         messages = list(queue.pop())
         if messages:
@@ -196,12 +208,25 @@ class SoftwareConfigService(service.Service):
         return software_deployment_object.SoftwareDeployment.get_by_id(
             cnxt, sd.id)
 
-    def show_software_deployment(self, cnxt, deployment_id):
+    def check_software_deployment(self, cnxt, deployment_id, timeout):
+        def _check():
+            while True:
+                sd = self._show_software_deployment(cnxt, deployment_id)
+                if sd.status != rpc_api.SOFTWARE_DEPLOYMENT_IN_PROGRESS:
+                    return
+                yield
+        scheduler.TaskRunner(_check)(timeout=timeout)
+        sd = software_deployment_object.SoftwareDeployment.get_by_id(
+            cnxt, deployment_id)
+        return api.format_software_deployment(sd)
+
+    def _show_software_deployment(self, cnxt, deployment_id):
         sd = software_deployment_object.SoftwareDeployment.get_by_id(
             cnxt, deployment_id)
         if sd.status == rpc_api.SOFTWARE_DEPLOYMENT_IN_PROGRESS:
             c = sd.config.config
-            input_values = dict((i['name'], i['value']) for i in c['inputs'])
+            input_values = dict(swc_io.InputConfig(**i).input_data()
+                                for i in c[rpc_api.SOFTWARE_CONFIG_INPUTS])
             transport = input_values.get('deploy_signal_transport')
             if transport == 'TEMP_URL_SIGNAL':
                 sd = self._refresh_swift_software_deployment(
@@ -209,6 +234,10 @@ class SoftwareConfigService(service.Service):
             elif transport == 'ZAQAR_SIGNAL':
                 sd = self._refresh_zaqar_software_deployment(
                     cnxt, sd, input_values.get('deploy_queue_id'))
+        return sd
+
+    def show_software_deployment(self, cnxt, deployment_id):
+        sd = self._show_software_deployment(cnxt, deployment_id)
         return api.format_software_deployment(sd)
 
     def create_software_deployment(self, cnxt, server_id, config_id,
@@ -258,9 +287,10 @@ class SoftwareConfigService(service.Service):
             status_reasons[output_status_code] = _(
                 'Deployment exited with non-zero status code: %s'
             ) % details.get(output_status_code)
-            event_reason = 'deployment failed (%s)' % status_code
+            event_reason = 'deployment %s failed (%s)' % (deployment_id,
+                                                          status_code)
         else:
-            event_reason = 'deployment succeeded'
+            event_reason = 'deployment %s succeeded' % deployment_id
 
         for output in sd.config.config['outputs'] or []:
             out_key = output['name']
@@ -269,7 +299,7 @@ class SoftwareConfigService(service.Service):
                 if output.get('error_output', False):
                     status = rpc_api.SOFTWARE_DEPLOYMENT_FAILED
                     status_reasons[out_key] = details[out_key]
-                    event_reason = 'deployment failed'
+                    event_reason = 'deployment %s failed' % deployment_id
 
         for out_key in rpc_api.SOFTWARE_DEPLOYMENT_OUTPUTS:
             ov[out_key] = details.get(out_key)
